import java.awt.Image;
import java.awt.image.ImageObserver;
import java.awt.image.BufferedImage;

import java.awt.image.ColorModel;

public class BufferedImageFilter2AsImageFilter implements ImageFilter {
	private BufferedImageFilter2 bufImgFilter;
	private BufferedImage inImgBuffer;
	private BufferedImage outImgBuffer;
	
	public BufferedImageFilter2AsImageFilter(BufferedImageFilter2 bufImgFilter) {
		this.bufImgFilter = bufImgFilter;
		this.inImgBuffer = null;
		this.outImgBuffer = null;
	}
	
	public Image createFilteredImage(Image img) {
		int width = img.getWidth(null);
		int height = img.getHeight(null);
		
		BufferedImage bufImg;
		if ((img instanceof BufferedImage) && ((BufferedImage) img).getColorModel().equals(ColorModel.getRGBdefault())) {
			/*
			 * Optimized: no separate input buffer required
			 * 
			 * The original Image is a BufferedImage and is already
			 * in the native ARGB format.
			 * 
			 * XXX: This implementation for the optimized case is currently untested
			 */
			
			// Ensure that the internal output BufferedImage
			// is size-compatible with the image to be filtered
			if ((this.outImgBuffer == null) ||
				(this.outImgBuffer.getWidth() != width) ||
				(this.outImgBuffer.getHeight() != height))
			{
				this.inImgBuffer = null;	// dispose of cached input buffer
				this.outImgBuffer = new BufferedImage(
					width,
					height,
					BufferedImage.TYPE_INT_ARGB);
			}
			
			bufImg = (BufferedImage) img;
		} else {
			/*
			 * General case: either...
			 * 1) Must allocate separate input buffer because the original
			 *    Image is not a BufferedImage and thus its pixels can't be read
			 * 2) Allocating a separate input buffer in the native ARGB
			 *    format and processing that buffer would be faster than
			 *    processing the original Image (which is a BufferedImage
			 *    but is in a non-native format)
			 */
			
			// Ensure that the internal input/output BufferedImages
			// are size-compatible with the image to be filtered
			if ((this.inImgBuffer == null) ||
				(this.inImgBuffer.getWidth() != width) ||
				(this.inImgBuffer.getHeight() != height))
			{
				this.inImgBuffer = new BufferedImage(
					width,
					height,
					BufferedImage.TYPE_INT_ARGB);
				this.outImgBuffer = new BufferedImage(
					width,
					height,
					BufferedImage.TYPE_INT_ARGB);
			}
			
			// Copy the data from the original image into the BufferedImage
			this.inImgBuffer.getGraphics().drawImage(img, 0, 0, (ImageObserver) null);
			
			bufImg = this.inImgBuffer;
		}
		
		// Filter the image
		bufImgFilter.filter(bufImg, outImgBuffer);
		
		// Return the filtered image
		return outImgBuffer;
	}
	
	public String toString() {
		return bufImgFilter.toString();
	}
}