/**
 * Filters that have adjustable parameters should implement this class
 * in order to make those parameters adjustable by the GUI.
 */
public interface ParameterizedFilter {
	/**
	 * Returns the set of adjustable parameters of this filter.
	 * The first time this method is invoked, the returned filter
	 * parameters should be initialized to their default initial
	 * values.
	 * <p>
	 * This method should always return the <i>same</i> set of
	 * parameters across multiple invocations.
	 * 
	 * @return	the set of parameters for this filter,
	 * 			or <tt>null</tt> if there are none.
	 */
	public FilterParam[] getParams();
}